@@ -31,11 +31,120 @@
 #include <Lum/String.h>
 #include <Lum/Table.h>
 
+class RangeDialog : public Lum::Dialog
+{
+private:
+  Lum::Model::StringRef startModel;
+  Lum::Model::StringRef endModel;
+
+  Lum::Model::ActionRef okAction;
+
+  bool                  hasResult;
+  size_t                start;
+  size_t                end;
+
+public:
+  RangeDialog()
+   : startModel(new Lum::Model::String(L"1")),
+     endModel(new Lum::Model::String(L"6")),
+     okAction(new Lum::Model::Action()),
+     hasResult(false)
+  {
+    Observe(startModel);
+    Observe(endModel);
+    Observe(okAction);
+    Observe(GetClosedAction());
+  }
+
+  void PreInit()
+  {
+    Lum::String *s,*e;
+
+    s=Lum::String::Create(startModel,5,true,false);
+    s->SetAlignment(Lum::String::right);
+    s->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typePosDec));
+    e=Lum::String::Create(endModel,5,true,false);
+    e->SetAlignment(Lum::String::right);
+    e->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typePosDec));
+
+    SetMain(Lum::VPanel::Create(false,false)
+            ->Add(Lum::Label::Create(true,false)
+                  ->AddLabel(L"Number range:",
+                             Lum::HPanel::Create(true,false)
+                             ->Add(s)
+                             ->AddSpace()
+                             ->Add(e)))
+            ->AddSpace()
+            ->Add(Lum::ButtonRow::CreateOkCancel(okAction,GetClosedAction(),true,false)));
+  }
+
+  bool HasResult() {
+    return hasResult;
+  }
+
+  void GetResult(size_t &start, size_t& end)
+  {
+    assert(hasResult);
+
+    start=this->start;
+    end=this->end;
+  }
+
+  void Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
+  {
+    if (model==okAction && okAction->IsFinished()) {
+      hasResult=true;
+      Exit();
+    }
+    else if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
+      Exit();
+    }
+    else if (model==startModel || model==endModel) {
+      if (Lum::Base::WStringToNumber(*startModel,start) &&
+          Lum::Base::WStringToNumber(*endModel,end) &&
+          end>=start) {
+        okAction->Enable();
+      }
+      else {
+        okAction->Disable();
+      }
+    }
+
+    Lum::Dialog::Resync(model,msg);
+  }
+
+  static bool GetRange(Lum::OS::Window* parent, size_t& start, size_t& end)
+  {
+    RangeDialog *dlg;
+    bool        res;
+
+    dlg=new RangeDialog();
+    dlg->SetParent(parent);
+    dlg->SetTitle(L""/*caption*/);
+
+    if (dlg->Open()) {
+      dlg->EventLoop();
+      dlg->Close();
+    }
+
+    res=dlg->HasResult();
+
+    if (res) {
+      dlg->GetResult(start,end);
+    }
+
+    delete dlg;
+
+    return res;
+  }
+};
+
 DiceEdit::DiceEdit(Dice& dice)
  : dice(dice),
    valuesModel(new ValuesModel(dice.values)),
    selection(new Lum::Model::SingleLineSelection()),
    addAction(new Lum::Model::Action()),
+   addNumbersAction(new Lum::Model::Action()),
    editAction(new Lum::Model::Action()),
    removeAction(new Lum::Model::Action())
 {
@@ -45,6 +154,7 @@ DiceEdit::DiceEdit(Dice& dice)
   Observe(selection);
 
   Observe(addAction);
+  Observe(addNumbersAction);
   Observe(editAction);
   Observe(removeAction);
 
@@ -79,6 +189,8 @@ void DiceEdit::PreInit()
   horiz->Add(Lum::VPanel::Create(false,true)
              ->Add(Lum::Button::Create(_(L"BTN_ADD",L"_Add"),addAction,true,false))
              ->AddSpace()
+             ->Add(Lum::Button::Create(_(L"BTN_ADD_NUMBERS",L"Add _numbers"),addNumbersAction,true,false))
+             ->AddSpace()
              ->Add(Lum::Button::Create(_(L"BTN_UPDATE",L"_Edit"),editAction,true,false))
              ->AddSpace()
              ->Add(Lum::Button::Create(_(L"BTN_REMOVE",L"_Remove"),removeAction,true,false)));
@@ -121,9 +233,26 @@ void DiceEdit::Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
 
       v.value=*value;
       valuesModel->Append(v);
-    }
 
-    configurationChanged=true;
+      configurationChanged=true;
+    }
+  }
+  else if (model==addNumbersAction && addNumbersAction->IsFinished()) {
+    bool   result=false;
+    size_t start=0;
+    size_t end=0;
+
+    result=RangeDialog::GetRange(GetWindow(),start,end);
+
+    if (result) {
+      for (size_t i=start; i<=end; i++) {
+        Value v;
+
+        v.value=Lum::Base::NumberToWString(i);
+        valuesModel->Append(v);
+      }
+      configurationChanged=true;
+    }
   }
   else if (model==editAction && editAction->IsFinished()) {
     assert(selection->HasSelection());
@@ -140,9 +269,9 @@ void DiceEdit::Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
     if (result) {
       valuesModel->GetEntry(selection->GetLine()).value=*value;
       valuesModel->RedrawRow(selection->GetLine());
-    }
 
-    configurationChanged=true;
+      configurationChanged=true;
+    }
   }
   else if (model==removeAction && removeAction->IsFinished()) {
     assert(selection->HasSelection());
